@@ -77,17 +77,19 @@ mcmc.transform_dictionary["tglitch"][
 t1 = time.time()
 mcmc.run(save_loudest=False)  # uses CFSv2 which doesn't support glitch parameters
 dT = time.time() - t1
+mcmc.print_summary()
+
 fig_and_axes = gridcorner._get_fig_and_axes(4, 2, 0.05)
 mcmc.plot_corner(
     label_offset=0.25,
-    truths=[0, 0, 0, 0],
+    truths={"F0": F0, "F1": F1, "delta_F0": delta_F0, "tglitch": tstart + dtglitch},
     fig_and_axes=fig_and_axes,
     quantiles=(0.16, 0.84),
     hist_kwargs=dict(lw=1.5, zorder=-1),
     truth_color="C3",
 )
 
-mcmc.print_summary()
+mcmc.plot_cumulative_max()
 
 print(("Prior widths =", F0_width, F1_width))
 print(("Actual run time = {}".format(dT)))
