@@ -20,6 +20,7 @@ from pyfstat.core import (
     SemiCoherentSearch,
     tqdm,
     args,
+    DeprecatedClass,
 )
 import lalpulsar
 import lal
@@ -167,9 +168,18 @@ class GridSearch(BaseSearchClass):
         if len(x) == 1:
             return np.array(x)
         elif len(x) == 3 and self.input_arrays is False:
-            return np.arange(x[0], x[1], x[2])
+            # This used to be
+            # return np.arange(x[0], x[1], x[2])
+            # but according to the numpy docs:
+            # "When using a non-integer step, such as 0.1,
+            # the results will often not be consistent.
+            # It is better to use numpy.linspace for these cases."
+            # and indeed it sometimes included the end point, sometimes didn't
+            return np.linspace(
+                x[0], x[1], num=int((x[1] - x[0]) / x[2]) + 1, endpoint=True
+            )
         else:
-            logging.info("Using tuple as is")
+            logging.info("Using tuple of length {:d} as is.".format(len(x)))
             return np.array(x)
 
     def get_input_data_array(self):
@@ -233,6 +243,30 @@ class GridSearch(BaseSearchClass):
         # need to convert any "None" entries in input_data array safely to 0s
         # to make np.allclose() work reliably
         new_data = np.nan_to_num(self.input_data.astype(np.float64))
+        if np.shape(old_data)[0] != np.shape(new_data)[0]:
+            logging.info(
+                "Old data found in '{:s}', but differs"
+                " in length ({:d} points in file, {:d} points requested);"
+                " continuing with grid search.".format(
+                    self.out_file, np.shape(old_data)[0], np.shape(new_data)[0]
+                )
+            )
+            return False
+        if np.shape(old_data)[1] < np.shape(new_data)[1]:
+            logging.info(
+                "Old data found in '{:s}', but has less columns ({:d})"
+                " than new input parameters grid ({:d});"
+                " continuing with grid search.".format(
+                    self.out_file, np.shape(old_data)[1], np.shape(new_data)[1]
+                )
+            )
+            return False
+        # not yet explicitly testing the case of
+        # np.shape(old_data)[1] >= np.shape(new_data)[1]
+        # because output file can have detstat and post-proc quantities
+        # added and hence have different number of dimensions
+        # (this could in principle be cleverly predicted at this point)
+        # and the np.allclose() check should safely catch those situations
         rtol, atol = self._get_tolerance_from_savetxt_fmt()
         column_matches = [
             np.allclose(
@@ -274,6 +308,11 @@ class GridSearch(BaseSearchClass):
             self.inititate_search_object()
 
         data = []
+        logging.info(
+            "Running search over a total of {:d} grid points...".format(
+                np.shape(iterable)[0]
+            )
+        )
         for vals in tqdm(iterable, total=getattr(self, "total_iterations", None)):
             detstat = self.search.get_det_stat(*vals)
             thisCand = list(vals) + [detstat]
@@ -722,6 +761,11 @@ class TransientGridSearch(GridSearch):
                 " results to {}*.dat".format(self.tCWfilebase)
             )
         self.timingFstatMap = 0.0
+        logging.info(
+            "Running search over a total of {:d} grid points...".format(
+                np.shape(self.input_data)[0]
+            )
+        )
         for vals in tqdm(self.input_data):
             detstat = self.search.get_det_stat(*vals)
             windowRange = getattr(self.search, "windowRange", None)
@@ -810,7 +854,7 @@ class TransientGridSearch(GridSearch):
             self.search.__del__()
 
 
-class SliceGridSearch(GridSearch):
+class SliceGridSearch(GridSearch, DeprecatedClass):
     """ Slice gridded search using ComputeFstat """
 
     @helper_functions.initializer
@@ -983,7 +1027,7 @@ class SliceGridSearch(GridSearch):
             return fig, axes
 
 
-class GridUniformPriorSearch:
+class GridUniformPriorSearch(DeprecatedClass):
     @helper_functions.initializer
     def __init__(
         self,
@@ -1143,7 +1187,7 @@ class GridGlitchSearch(GridSearch):
         return fmt
 
 
-class SlidingWindow(GridSearch):
+class SlidingWindow(GridSearch, DeprecatedClass):
     @helper_functions.initializer
     def __init__(
         self,
@@ -1281,7 +1325,7 @@ class SlidingWindow(GridSearch):
             return ax
 
 
-class FrequencySlidingWindow(GridSearch):
+class FrequencySlidingWindow(GridSearch, DeprecatedClass):
     """ A sliding-window search over the Frequency """
 
     @helper_functions.initializer
@@ -1442,7 +1486,7 @@ class FrequencySlidingWindow(GridSearch):
             return ax
 
 
-class EarthTest(GridSearch):
+class EarthTest(GridSearch, DeprecatedClass):
     """ """
 
     tex_labels = {
@@ -1695,7 +1739,7 @@ class EarthTest(GridSearch):
         plt.close(fig)
 
 
-class DMoff_NO_SPIN(GridSearch):
+class DMoff_NO_SPIN(GridSearch, DeprecatedClass):
     """ DMoff test using SSBPREC_NO_SPIN """
 
     @helper_functions.initializer
