@@ -714,13 +714,33 @@ def get_dictionary_from_lines(lines, comments, raise_error):
     return d
 
 
+def get_predict_fstat_parameters_from_dict(signal_parameters):
+    """
+    Given a dictionary with arbitrary signal parameters,
+    return only those ones required by helper_functions.predict_fstat
+    (Freq, Alpha, Delta, h0, cosi, psi)
+
+    Parameters
+    ----------
+    signal_parameters: dict
+        Dictionary containing at least the signal parameters required by
+        helper_functions.predict_fstat. This dictionary's keys must follow
+        the PyFstat convention (F0 instead of Freq).
+    """
+    predict_fstat_params = {
+        key: signal_parameters[key]
+        for key in ["F0", "Alpha", "Delta", "h0", "cosi", "psi"]
+    }
+    return predict_fstat_params
+
+
 def predict_fstat(
     h0=None,
     cosi=None,
     psi=None,
     Alpha=None,
     Delta=None,
-    Freq=None,
+    F0=None,
     sftfilepattern=None,
     timestampsFiles=None,
     minStartTime=None,
@@ -733,7 +753,6 @@ def predict_fstat(
     transientWindowType="none",
     transientStartTime=None,
     transientTau=None,
-    **kwargs
 ):
     """Wrapper to lalapps_PredictFstat
 
@@ -741,7 +760,7 @@ def predict_fstat(
     ----------
     h0, cosi, psi, Alpha, Delta : float
         Signal properties, see `lalapps_PredictFstat --help` for more info.
-    Freq: float or None
+    F0: float or None
         Only needed for noise floor estimation when given sftfilepattern
         but assumeSqrtSX is None.
     sftfilepattern : str or None
@@ -757,9 +776,12 @@ def predict_fstat(
         Comma-separated list of detectors.
         Ignored if sftfilepattern is given,
         required if it is not.
-    assumeSqrtSX : float or str or None
+    assumeSqrtSX : float or str
         Assume stationary per-detector noise-floor instead of estimating from SFTs.
-        For multiple detectors: comma-separated list
+        Single float or str value: use same for all IFOs.
+        Comma-separated string: must match len(detectors)
+        and/or the data in sftfilepattern.
+        Detectors will be paired to list elements following alphabetical order.
         Required if sftfilepattern is not given,
         optional if it is.
     tempory_filename : str
@@ -828,14 +850,14 @@ def predict_fstat(
             if duration is not None:
                 cl_pfs.append("--maxStartTime={}".format(minStartTime + duration))
         if assumeSqrtSX is None:
-            if Freq is None:
+            if F0 is None:
                 raise ValueError(
                     "With sftfilepattern but without assumeSqrtSX,"
-                    " we need Freq to estimate noise floor."
+                    " we need F0 to estimate noise floor."
                 )
-            cl_pfs.append("--Freq={}".format(Freq))
+            cl_pfs.append("--Freq={}".format(F0))
     if assumeSqrtSX is not None:
-        if assumeSqrtSX <= 0:
+        if np.any([s <= 0 for s in parse_list_of_numbers(assumeSqrtSX)]):
             raise ValueError("assumeSqrtSX must be >0!")
         cl_pfs.append("--assumeSqrtSX={}".format(assumeSqrtSX))
 
@@ -861,3 +883,21 @@ def predict_fstat(
     twoF_sigma = float(d["twoF_sigma"])
     os.remove(tempory_filename)
     return twoF_expected, twoF_sigma
+
+
+def parse_list_of_numbers(val):
+    """Convert a number, list of numbers or comma-separated str into a list of numbers.
+
+    This is useful e.g. for sqrtSX inputs where the user can be expected
+    to try either type of input.
+    """
+    try:
+        out = list(
+            map(float, val.split(",") if hasattr(val, "split") else np.atleast_1d(val))
+        )
+    except Exception as e:
+        raise ValueError(
+            f"Could not parse '{val}' as a number or list of numbers."
+            f" Got exception: {e}."
+        )
+    return out
