@@ -2,6 +2,7 @@
 
 
 import numpy as np
+import functools
 import logging
 import os
 import glob
@@ -23,6 +24,100 @@ class KeyboardInterruptError(Exception):
     pass
 
 
+class InjectionParametersGenerator:
+    """
+    Draw injection parameter samples from the specified priors and return
+    them in the proper format.
+    """
+
+    def __init__(self, priors=None, seed=None):
+        """
+        priors: dict
+            Each key refers to one of the signal's parameters (following the PyFstat convention).
+            Priors can be given as values in three formats (by order of evaluation):
+                1) Callable without required arguments
+                    {"ParameterA": np.random.uniform}
+                2) Dict containing numpy.random distribution as key and kwargs in a dict as value
+                    {"ParameterA": {"uniform": {"low": 0, "high":1}}}
+                3) Constant value to be returned as is
+                    {"ParameterA": 1.0}
+        seed:
+            `seed` argument to be feed to numpy.random.default_rng.
+        """
+        self.set_seed(seed)
+        self.set_priors(priors or {})
+
+    def set_priors(self, new_priors):
+        """Set priors to draw parameter space points from """
+        if type(new_priors) is not dict:
+            raise ValueError(
+                "new_priors is not a dict type.\nPlease, check "
+                "this class' docstring to learn about the expected format: "
+                f"{self.__init__.__doc__}"
+            )
+
+        self.priors = {}
+        self._update_priors(new_priors)
+
+    def set_seed(self, seed):
+        self.seed = seed
+        self._rng = np.random.default_rng(self.seed)
+
+    def _update_priors(self, new_priors):
+        for parameter_name, parameter_prior in new_priors.items():
+            if callable(parameter_prior):
+                self.priors[parameter_name] = parameter_prior
+            elif isinstance(parameter_prior, dict):
+                rng_function_name = next(iter(parameter_prior))
+                rng_function = getattr(self._rng, rng_function_name)
+                rng_kwargs = parameter_prior[rng_function_name]
+                self.priors[parameter_name] = functools.partial(
+                    rng_function, **rng_kwargs
+                )
+            else:  # Assume it is something to be returned as is
+                self.priors[parameter_name] = functools.partial(
+                    lambda x: x, parameter_prior
+                )
+
+    def draw(self):
+        injection_parameters = {
+            parameter_name: parameter_prior()
+            for parameter_name, parameter_prior in self.priors.items()
+        }
+        return injection_parameters
+
+    def __call__(self):
+        return self.draw()
+
+
+class AllSkyInjectionParametersGenerator(InjectionParametersGenerator):
+    """Like InjectionParametersGenerator, but with hardcoded priors to perform
+    all sky searches. It assumes 1) PyFstat notation and 2) Equatorial coordinates"""
+
+    def set_priors(self, new_priors):
+        self._check_if_updating_sky_priors(new_priors)
+        super().set_priors({**new_priors, **self.restricted_priors})
+
+    def set_seed(self, seed):
+        super().set_seed(seed)
+        self.restricted_priors = {
+            # This is required because numpy has no arcsin distro
+            "Alpha": lambda: self._rng.uniform(low=0.0, high=2 * np.pi),
+            "Delta": lambda: np.arcsin(self._rng.uniform(low=-1.0, high=1.0)),
+        }
+
+    def _check_if_updating_sky_priors(self, new_priors):
+        if any(
+            restricted_key in new_priors
+            for restricted_key in self.restricted_priors.keys()
+        ):
+            logging.warning(
+                "Ignoring specified sky priors (Alpha, Delta)."
+                "This class is explicitly coded to prevent that from happening. Please, restore "
+                "to InjectionParametersGenerator if that's really what you want to do."
+            )
+
+
 class Writer(BaseSearchClass):
     """ Instance object for generating SFTs """
 
@@ -97,9 +192,10 @@ class Writer(BaseSearchClass):
             frequency evolution and amplitude parameters for injection
             if h0==None or h0==0, these are all ignored
             if h0>0, then Alpha, Delta, cosi need to be set explicitly
-        Tsft: float
-            the sft duration
-        noiseSFTs: str
+        Tsft: int
+            The SFT duration in seconds.
+            Will be ignored if noiseSFTs are given.
+        noiseSFTs: str or None
             SFT on top of which signals will be injected.
             If not None, additional constraints can be applied using the arguments
             tstart and duration.
@@ -154,16 +250,20 @@ class Writer(BaseSearchClass):
         self.duration = int(self.duration)
 
         IFOs = self.detectors.split(",")
-        numSFTs = len(IFOs) * [int(float(self.duration) / self.Tsft)]
+        # when duration is not a multiple of Tsft, to match MFDv5
+        # we need to round the number *up*
+        # and also include the overlapping bit at the end in the duration
+        numSFTs = int(np.ceil(float(self.duration) / self.Tsft))  # per IFO
+        effective_duration = numSFTs * self.Tsft
 
         self.sftfilenames = [
             lalpulsar.OfficialSFTFilename(
                 dets[0],
                 dets[1],
-                numSFTs[ind],
+                numSFTs,
                 self.Tsft,
                 self.tstart,
-                self.duration,
+                effective_duration,
                 self.label,
             )
             for ind, dets in enumerate(IFOs)
@@ -223,13 +323,13 @@ class Writer(BaseSearchClass):
         # Get the "overall" values of the search
         Tsft = np.unique(Tsft)
         if len(Tsft) != 1:
-            raise ValueError("SFTs contain different basetimes: {}".format(Tsft))
-        elif Tsft[0] != self.Tsft:
-            raise ValueError(
-                "SFT basetime {} differs from input base time {}".format(
-                    Tsft[0], self.Tsft
-                )
+            raise ValueError(f"SFTs contain different basetimes: {Tsft}")
+        if Tsft[0] != self.Tsft:
+            logging.warning(
+                f"Overwriting self.Tsft={self.Tsft}"
+                f" with value {Tsft[0]} read from noiseSFTs."
             )
+        self.Tsft = Tsft[0]
         self.tstart = min(tstart)
         self.duration = max(tend) - self.tstart
         self.detectors = ",".join(IFOs)
@@ -386,15 +486,11 @@ class Writer(BaseSearchClass):
         content = self.get_single_config_line(0)
 
         if verbose:
-            logging.info(
-                "Writing the following injection parameters"
-                " to config file {:s}:".format(self.config_file_name)
-            )
-            logging.info(content)
-        else:
-            logging.info("Writing config file {:s}...".format(self.config_file_name))
+            logging.info("Injection parameters:")
+            logging.info(content.rstrip("\n"))
 
         if self.check_if_cff_file_needs_rewriting(content):
+            logging.info("Writing config file: {:s}".format(self.config_file_name))
             config_file = open(self.config_file_name, "w+")
             config_file.write(content)
             config_file.close()
@@ -530,8 +626,8 @@ class Writer(BaseSearchClass):
         except OSError:
             pass
 
-        cl_mfd = []
-        cl_mfd.append("lalapps_Makefakedata_v5")
+        mfd = "lalapps_Makefakedata_v5"
+        cl_mfd = [mfd]
         cl_mfd.append("--outSingleSFT=TRUE")
         cl_mfd.append('--outSFTdir="{}"'.format(self.outdir))
         cl_mfd.append('--outLabel="{}"'.format(self.label))
@@ -543,7 +639,9 @@ class Writer(BaseSearchClass):
                 "to produce noiseSFTs."
             )
         elif self.noiseSFTs is not None:
-            if self.sqrtSX and self.sqrtSX > 0.0:
+            if self.sqrtSX and np.any(
+                [s > 0 for s in helper_functions.parse_list_of_numbers(self.sqrtSX)]
+            ):
                 logging.warning(
                     "In addition to using noiseSFTs, you are adding "
                     "Gaussian noise with sqrtSX={} "
@@ -586,7 +684,13 @@ class Writer(BaseSearchClass):
         check_ok = self.check_cached_data_okay_to_use(cl_mfd)
         if check_ok is False:
             helper_functions.run_commandline(cl_mfd)
-            logging.info("Successfully wrote SFTs to: {}".format(self.sftfilepath))
+            if np.all([os.path.isfile(f) for f in self.sftfilepath.split(";")]):
+                logging.info(f"Successfully wrote SFTs to: {self.sftfilepath}")
+            else:
+                raise RuntimeError(
+                    f"It seems we successfully ran {mfd},"
+                    f" but did not get the expected SFT file path(s): {self.sftfilepath}."
+                )
 
     def predict_fstat(self, assumeSqrtSX=None):
         """ Wrapper to lalapps_PredictFstat """
@@ -596,7 +700,7 @@ class Writer(BaseSearchClass):
             psi=self.psi,
             Alpha=self.Alpha,
             Delta=self.Delta,
-            Freq=self.F0,
+            F0=self.F0,
             sftfilepattern=self.sftfilepath,
             minStartTime=self.tstart,
             duration=self.duration,
@@ -937,13 +1041,11 @@ transientTau = {:10.0f}\n"""
             content += line
 
         if verbose:
-            logging.info(
-                "Writing the following injection parameters"
-                " to config file {:s}:".format(self.config_file_name)
-            )
-            logging.info(content)
+            logging.info("Injection parameters:")
+            logging.info(content.rstrip("\n"))
 
         if self.check_if_cff_file_needs_rewriting(content):
+            logging.info("Writing config file: {:s}".format(self.config_file_name))
             config_file = open(self.config_file_name, "w+")
             config_file.write(content)
             config_file.close()
