@@ -31,16 +31,6 @@ class BaseForTestsWithOutdir(unittest.TestCase):
                 logging.warning("{} not removed after tests".format(self.outdir))
 
 
-default_signal_params = {
-    "F0": 30,
-    "F1": -1e-10,
-    "F2": 0,
-    "h0": 5,
-    "cosi": 0,
-    "Alpha": 5e-3,
-    "Delta": 1.2,
-}
-
 default_Writer_params = {
     "label": "test",
     "sqrtSX": 1,
@@ -52,9 +42,21 @@ default_Writer_params = {
     "SFTWindowBeta": 0.001,
     "randSeed": 42,
     "Band": None,
-    **default_signal_params,
 }
 
+default_signal_params = {
+    "F0": 30,
+    "F1": -1e-10,
+    "F2": 0,
+    "h0": 5.0,
+    "cosi": 0,
+    "psi": 0,
+    "phi": 0,
+    "Alpha": 5e-1,
+    "Delta": 1.2,
+}
+
+
 default_binary_params = {
     "period": 45 * 24 * 3600.0,
     "asini": 10.0,
@@ -82,7 +84,7 @@ class BaseForTestsWithData(BaseForTestsWithOutdir):
         # if we directly set any options as self.xy = 1 here,
         # then values set for derived classes may get overwritten,
         # so use a default dict and only insert if no value previous set
-        for key, val in default_Writer_params.items():
+        for key, val in {**default_Writer_params, **default_signal_params}.items():
             if not hasattr(self, key):
                 setattr(self, key, val)
         self.tref = self.tstart
@@ -112,9 +114,88 @@ class BaseForTestsWithData(BaseForTestsWithOutdir):
         self.search_ranges = {key: [getattr(self, key)] for key in self.search_keys}
 
 
+class TestInjectionParametersGenerator(BaseForTestsWithOutdir):
+    label = "TestInjectionParametersGenerator"
+    class_to_test = pyfstat.InjectionParametersGenerator
+
+    def test_numpy_priors(self):
+        numpy_priors = {
+            "ParameterA": {"uniform": {"low": 0.0, "high": 0.0}},
+            "ParameterB": {"uniform": {"low": 1.0, "high": 1.0}},
+        }
+        InjectionGenerator = self.class_to_test(numpy_priors)
+
+        parameters = InjectionGenerator.draw()
+        self.assertTrue(parameters["ParameterA"] == 0.0)
+        self.assertTrue(parameters["ParameterB"] == 1.0)
+
+    def test_callable_priors(self):
+        callable_priors = {"ParameterA": lambda: 0.0, "ParameterB": lambda: 1.0}
+        InjectionGenerator = self.class_to_test(callable_priors)
+
+        parameters = InjectionGenerator.draw()
+        self.assertTrue(parameters["ParameterA"] == 0.0)
+        self.assertTrue(parameters["ParameterB"] == 1.0)
+
+    def test_constant_priors(self):
+        constant_priors = {"ParameterA": 0.0, "ParameterB": 1.0}
+        InjectionGenerator = self.class_to_test(constant_priors)
+
+        parameters = InjectionGenerator.draw()
+        self.assertTrue(parameters["ParameterA"] == 0.0)
+        self.assertTrue(parameters["ParameterB"] == 1.0)
+
+    def test_rng_seed(self):
+        a_seed = 420
+
+        samples = []
+        for i in range(2):
+            InjectionGenerator = self.class_to_test(
+                priors={"ParameterA": {"normal": {"loc": 0, "scale": 1}}}, seed=a_seed
+            )
+            samples.append(InjectionGenerator.draw())
+        self.assertTrue(samples[0]["ParameterA"] == samples[1]["ParameterA"])
+
+    def test_rng_generation(self):
+        InjectionGenerator = self.class_to_test(
+            priors={"ParameterA": {"normal": {"loc": 0, "scale": 0.01}}}
+        )
+        samples = [InjectionGenerator.draw()["ParameterA"] for i in range(100)]
+        mean = np.mean(samples)
+        self.assertTrue(np.abs(mean) < 0.1)
+
+
+class TestAllSkyInjectionParametersGenerator(TestInjectionParametersGenerator):
+    label = "TestAllInjectionParametersGenerator"
+
+    class_to_test = pyfstat.AllSkyInjectionParametersGenerator
+
+    def test_rng_seed(self):
+        a_seed = 420
+
+        samples = []
+        for i in range(2):
+            InjectionGenerator = self.class_to_test(seed=a_seed)
+            samples.append(InjectionGenerator.draw())
+        self.assertTrue(samples[0]["Alpha"] == samples[1]["Alpha"])
+        self.assertTrue(samples[0]["Delta"] == samples[1]["Delta"])
+
+    def test_rng_generation(self):
+        InjectionGenerator = self.class_to_test()
+        ra_samples = [
+            InjectionGenerator.draw()["Alpha"] / np.pi - 1.0 for i in range(500)
+        ]
+        dec_samples = [
+            InjectionGenerator.draw()["Delta"] * 2.0 / np.pi for i in range(500)
+        ]
+        self.assertTrue(np.abs(np.mean(ra_samples)) < 0.1)
+        self.assertTrue(np.abs(np.mean(dec_samples)) < 0.1)
+
+
 class TestWriter(BaseForTestsWithData):
     label = "TestWriter"
     writer_class_to_test = pyfstat.Writer
+    signal_parameters = default_signal_params
 
     def test_make_cff(self):
         self.Writer.make_cff(verbose=True)
@@ -124,16 +205,17 @@ class TestWriter(BaseForTestsWithData):
 
     def test_run_makefakedata(self):
         self.Writer.make_data(verbose=True)
+        numSFTs = int(np.ceil(self.duration / self.Tsft))
         expected_outfile = os.path.join(
             self.Writer.outdir,
             "{:1s}-{:d}_{:2s}_{:d}SFT_{:s}-{:d}-{:d}.sft".format(
                 self.detectors[0],
-                int(self.duration / self.Tsft),
+                numSFTs,
                 self.detectors,
                 self.Tsft,
                 self.Writer.label,
                 self.Writer.tstart,
-                self.Writer.duration,
+                numSFTs * self.Tsft,
             ),
         )
         self.assertTrue(os.path.isfile(expected_outfile))
@@ -165,7 +247,7 @@ class TestWriter(BaseForTestsWithData):
 
     def test_noise_sfts(self):
         randSeed = 69420
-        detectors = "L1,H1"
+        detectors = "H1,L1"
 
         # create SFTs with both noise and a signal in them
         noise_and_signal_writer = self.writer_class_to_test(
@@ -179,7 +261,8 @@ class TestWriter(BaseForTestsWithData):
             SFTWindowType=self.SFTWindowType,
             SFTWindowBeta=self.SFTWindowBeta,
             sqrtSX=self.sqrtSX,
-            **default_signal_params,
+            Band=0.5,
+            **self.signal_parameters,
         )
         noise_and_signal_writer.make_data(verbose=True)
         # FIXME: here and everywhere below,
@@ -187,11 +270,13 @@ class TestWriter(BaseForTestsWithData):
         times, freqs, data = pyfstat.helper_functions.get_sft_array(
             noise_and_signal_writer.sftfilepath
         )
+        # Store the maximum SFT values for later comparison
         max_values_noise_and_signal = np.max(data, axis=0)
         max_freqs_noise_and_signal = freqs[np.argmax(data, axis=0)]
-        self.assertTrue(len(times) == self.duration / self.Tsft)
-        # with signal: all SFTs should peak at same freq
-        self.assertTrue(len(np.unique(max_freqs_noise_and_signal)) == 1)
+        self.assertTrue(len(times) == int(np.ceil(self.duration / self.Tsft)))
+        # FIXME: CW signals don't have to peak at the same frequency, but there
+        # are some consistency criteria which may be useful to implement here.
+        # self.assertTrue(len(np.unique(max_freqs_noise_and_signal)) == 1)
 
         # create noise-only SFTs
         noise_writer = self.writer_class_to_test(
@@ -205,8 +290,8 @@ class TestWriter(BaseForTestsWithData):
             SFTWindowType=self.SFTWindowType,
             SFTWindowBeta=self.SFTWindowBeta,
             sqrtSX=self.sqrtSX,
-            h0=0,
-            F0=self.F0,
+            Band=0.5,
+            F0=self.signal_parameters["F0"],
         )
         noise_writer.make_data(verbose=True)
         times, freqs, data = pyfstat.helper_functions.get_sft_array(
@@ -223,14 +308,10 @@ class TestWriter(BaseForTestsWithData):
         add_signal_writer = self.writer_class_to_test(
             label="test_noiseSFTs_add_signal",
             outdir=self.outdir,
-            duration=None,
-            Tsft=self.Tsft,
-            tstart=None,
             SFTWindowType=self.SFTWindowType,
             SFTWindowBeta=self.SFTWindowBeta,
-            sqrtSX=0,
             noiseSFTs=noise_writer.sftfilepath,
-            **default_signal_params,
+            **self.signal_parameters,
         )
         add_signal_writer.make_data(verbose=True)
         times, freqs, data = pyfstat.helper_functions.get_sft_array(
@@ -259,7 +340,7 @@ class TestWriter(BaseForTestsWithData):
             SFTWindowBeta=self.SFTWindowBeta,
             sqrtSX=0,
             noiseSFTs=noise_writer.sftfilepath,
-            **default_signal_params,
+            **self.signal_parameters,
         )
         add_signal_writer_constr.make_data(verbose=True)
         times, freqs, data = pyfstat.helper_functions.get_sft_array(
@@ -352,22 +433,31 @@ class TestWriter(BaseForTestsWithData):
         self.assertTrue(os.path.isfile(expected_SFT_filepath))
 
 
+class TestWriterOtherTsft(TestWriter):
+    label = "TestWriterOtherTsft"
+    writer_class_to_test = pyfstat.Writer
+
+    @classmethod
+    def setUpClass(self):
+        self.Tsft = 1024
+        super().setUpClass()
+
+
 class TestBinaryModulatedWriter(TestWriter):
     label = "TestBinaryModulatedWriter"
     writer_class_to_test = pyfstat.BinaryModulatedWriter
+    signal_parameters = {**default_signal_params, **default_binary_params}
 
     def test_tp_parsing(self):
         this_writer = self.writer_class_to_test(
-            outdir=self.outdir,
-            **default_Writer_params,
-            **default_binary_params,
+            outdir=self.outdir, **default_Writer_params, **self.signal_parameters
         )
         this_writer.make_data()
 
         theta_prior = {
             key: value
             for key, value in default_signal_params.items()
-            if key not in ["h0", "cosi"]
+            if key not in ["h0", "cosi", "psi", "phi"]
         }
         theta_prior.update({key: value for key, value in default_binary_params.items()})
         theta_prior["tp"] = {
@@ -670,9 +760,10 @@ class TestComputeFstat(BaseForTestsWithData):
         self.assertTrue(np.abs(predicted_FS - FS_from_file) / FS_from_file < 0.3)
 
         injectSourcesdict = search.read_par(filename=injectSources)
-        injectSourcesdict["F0"] = injectSourcesdict["Freq"]
-        injectSourcesdict["F1"] = injectSourcesdict["f1dot"]
-        injectSourcesdict["F2"] = injectSourcesdict["f2dot"]
+        injectSourcesdict["F0"] = injectSourcesdict.pop("Freq")
+        injectSourcesdict["F1"] = injectSourcesdict.pop("f1dot")
+        injectSourcesdict["F2"] = injectSourcesdict.pop("f2dot")
+        injectSourcesdict["phi"] = injectSourcesdict.pop("phi0")
         search = pyfstat.ComputeFstat(
             tref=self.Writer.tref,
             assumeSqrtSX=1,
@@ -748,6 +839,61 @@ class TestComputeFstat(BaseForTestsWithData):
         )
         self.assertTrue(lnBSGL > 0)
 
+    def test_cumulative_twoF(self):
+        Nsft = 100
+        # not using any SFTs on disk
+        search = pyfstat.ComputeFstat(
+            tref=self.tref,
+            minStartTime=self.tstart,
+            maxStartTime=self.tstart + Nsft * self.Tsft,
+            detectors=self.detectors,
+            injectSqrtSX=self.sqrtSX,
+            injectSources=default_signal_params,
+            minCoverFreq=self.F0 - 0.1,
+            maxCoverFreq=self.F0 + 0.1,
+        )
+        start_time, taus, twoF_cumulative = search.calculate_twoF_cumulative(
+            self.Writer.F0,
+            self.Writer.F1,
+            self.Writer.F2,
+            self.Writer.Alpha,
+            self.Writer.Delta,
+            num_segments=Nsft + 1,
+        )
+        twoF = search.get_fullycoherent_twoF(
+            self.Writer.tstart,
+            self.Writer.tstart + taus[-1],
+            self.Writer.F0,
+            self.Writer.F1,
+            self.Writer.F2,
+            self.Writer.Alpha,
+            self.Writer.Delta,
+        )
+        reldiff = np.abs(twoF_cumulative[-1] - twoF) / twoF
+        print(
+            "2F from get_fullycoherent_twoF() is {:.4f}"
+            " while last value from calculate_twoF_cumulative() is {:.4f};"
+            " relative difference: {:g}".format(twoF, twoF_cumulative[-1], reldiff)
+        )
+        self.assertTrue(reldiff < 0.1)
+        idx = int(Nsft / 2)
+        partial_2F_expected = (taus[idx] / taus[-1]) * twoF
+        reldiff = (
+            np.abs(twoF_cumulative[idx] - partial_2F_expected) / partial_2F_expected
+        )
+        print(
+            "Middle 2F value from calculate_twoF_cumulative() is {:.4f}"
+            " while from duration ratio we'd expect {:.4f}*{:.4f}={:.4f};"
+            " relative difference: {:g}".format(
+                twoF_cumulative[idx],
+                taus[idx] / taus[-1],
+                twoF,
+                partial_2F_expected,
+                reldiff,
+            )
+        )
+        self.assertTrue(reldiff < 0.1)
+
 
 class TestComputeFstatNoNoise(BaseForTestsWithData):
     # FIXME: should be possible to merge into TestComputeFstat with smart
@@ -1224,8 +1370,8 @@ class TestMCMCSearch(BaseForMCMCSearchTests):
                 theta_prior=thetas[prior_choice],
                 tref=self.tref,
                 sftfilepattern=self.Writer.sftfilepath,
-                nsteps=[100, 100],
-                nwalkers=100,
+                nsteps=[20, 20],
+                nwalkers=20,
                 ntemps=2,
                 log10beta_min=-1,
             )
@@ -1404,9 +1550,12 @@ class TestMCMCTransientSearch(BaseForMCMCSearchTests):
 
 class TestGridSearch(BaseForTestsWithData):
     label = "TestGridSearch"
-    F0s = [29, 31, 0.1]
+    # Need to hand-pick values F0s here for the CFSv2 comparison:
+    # that code sometimes includes endpoints, sometimes not.
+    # For the F0s here, it happens to match our convention (include endpoint).
+    F0s = [29.999, 30.001, 1e-4]
     F1s = [-1e-10, 0, 1e-11]
-    Band = 2.5
+    Band = 0.5
 
     def test_grid_search(self):
         search = pyfstat.GridSearch(
@@ -1414,10 +1563,10 @@ class TestGridSearch(BaseForTestsWithData):
             self.outdir,
             self.Writer.sftfilepath,
             F0s=self.F0s,
-            F1s=[0],
-            F2s=[0],
-            Alphas=[0],
-            Deltas=[0],
+            F1s=[self.Writer.F1],
+            F2s=[self.Writer.F2],
+            Alphas=[self.Writer.Alpha],
+            Deltas=[self.Writer.Delta],
             tref=self.tref,
         )
         search.run()
@@ -1433,10 +1582,10 @@ class TestGridSearch(BaseForTestsWithData):
             self.outdir,
             self.Writer.sftfilepath,
             F0s=self.F0s,
-            F1s=[0],
-            F2s=[0],
-            Alphas=[0],
-            Deltas=[0],
+            F1s=[self.Writer.F1],
+            F2s=[self.Writer.F2],
+            Alphas=[self.Writer.Alpha],
+            Deltas=[self.Writer.Delta],
             tref=self.tref,
         )
         search.run()
@@ -1448,12 +1597,12 @@ class TestGridSearch(BaseForTestsWithData):
         CFSv2_loudest_file = os.path.join(self.outdir, "CFSv2_Fstat_loudest.txt")
         cl_CFSv2 = []
         cl_CFSv2.append("lalapps_ComputeFstatistic_v2")
-        cl_CFSv2.append("--Alpha 0 --Delta 0")
-        cl_CFSv2.append("--AlphaBand 0 --DeltaBand 0")
+        cl_CFSv2.append("--Alpha {} --AlphaBand 0".format(self.Alpha))
+        cl_CFSv2.append("--Delta {} --DeltaBand 0".format(self.Delta))
         cl_CFSv2.append("--Freq {}".format(self.F0s[0]))
-        cl_CFSv2.append("--f1dot 0 --f1dotBand 0 --df1dot 0")
         cl_CFSv2.append("--FreqBand {}".format(self.F0s[1] - self.F0s[0]))
         cl_CFSv2.append("--dFreq {}".format(self.F0s[2]))
+        cl_CFSv2.append("--f1dot {} --f1dotBand 0".format(self.F1))
         cl_CFSv2.append("--DataFiles " + self.Writer.sftfilepath)
         cl_CFSv2.append("--refTime {}".format(self.tref))
         earth_ephem, sun_ephem = pyfstat.helper_functions.get_ephemeris_files()
@@ -1488,10 +1637,10 @@ class TestGridSearch(BaseForTestsWithData):
             self.outdir,
             self.Writer.sftfilepath,
             F0s=self.F0s,
-            F1s=[0],
-            F2s=[0],
-            Alphas=[0],
-            Deltas=[0],
+            F1s=[self.Writer.F1],
+            F2s=[self.Writer.F2],
+            Alphas=[self.Writer.Alpha],
+            Deltas=[self.Writer.Delta],
             tref=self.tref,
             nsegs=2,
         )
@@ -1506,9 +1655,9 @@ class TestGridSearch(BaseForTestsWithData):
             self.Writer.sftfilepath,
             F0s=self.F0s,
             F1s=self.F1s,
-            F2s=[0.0],
-            Alphas=[0.0],
-            Deltas=[0.0],
+            F2s=[self.Writer.F2],
+            Alphas=[self.Writer.Alpha],
+            Deltas=[self.Writer.Delta],
             tref=self.tref,
             Lambda0=[30.0, 0.0, 0.0, 0.0],
         )
@@ -1522,9 +1671,9 @@ class TestGridSearch(BaseForTestsWithData):
             self.Writer.sftfilepath,
             F0s=self.F0s,
             F1s=self.F1s,
-            F2s=[0],
-            Alphas=[0],
-            Deltas=[0],
+            F2s=[self.Writer.F2],
+            Alphas=[self.Writer.Alpha],
+            Deltas=[self.Writer.Delta],
             tref=self.tref,
             tglitchs=[self.tref],
         )
@@ -1538,10 +1687,10 @@ class TestGridSearch(BaseForTestsWithData):
             self.outdir,
             self.Writer.sftfilepath,
             F0s=self.F0s,
-            F1=0,
-            F2=0,
-            Alpha=0,
-            Delta=0,
+            F1=self.Writer.F1,
+            F2=self.Writer.F2,
+            Alpha=self.Writer.Alpha,
+            Delta=self.Writer.Delta,
             tref=self.tref,
             minStartTime=self.Writer.tstart,
             maxStartTime=self.Writer.tend(),
@@ -1561,10 +1710,10 @@ class TestTransientGridSearch(BaseForTestsWithData):
             self.outdir,
             self.Writer.sftfilepath,
             F0s=self.F0s,
-            F1s=[0],
-            F2s=[0],
-            Alphas=[0],
-            Deltas=[0],
+            F1s=[self.Writer.F1],
+            F2s=[self.Writer.F2],
+            Alphas=[self.Writer.Alpha],
+            Deltas=[self.Writer.Delta],
             tref=self.tref,
             minStartTime=self.Writer.tstart,
             maxStartTime=self.Writer.tend(),
